package com.citygis.web.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.citygis.common.annotation.DataScope;
import com.citygis.common.core.domain.AjaxResult;
import com.citygis.web.domain.TabAccessFeedback;
import com.citygis.web.mapper.TabAccessFeedbackMapper;
import com.citygis.web.service.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.citygis.common.utils.SecurityUtils.*;

/**
 * <p>
 * 接入反馈表 服务实现类
 * </p>
 *
 * @author citygis
 * @since 2025-01-07
 */
@Service
public class TabAccessFeedbackServiceImpl extends ServiceImpl<TabAccessFeedbackMapper, TabAccessFeedback> implements ITabAccessFeedbackService {

    @Resource
    TabAccessFeedbackMapper tabAccessFeedbackMapper;

    @Override
    @DataScope(userAlias = "taf", deptAlias = "taf")
    public List<TabAccessFeedback> getTabAccessFeedbackList(TabAccessFeedback tabAccessFeedback) {

        if (tabAccessFeedback.getStates() != null) {
            Integer state = tabAccessFeedback.getState();
            List<Integer> states = new ArrayList<>();
            //正式运行
            if (state == 1) {
                states.add(5);
                states.add(6);
            } else if (state == 2) {//申请反馈
                states.add(1);
                states.add(2);
                states.add(3);
                states.add(4);
            }
            tabAccessFeedback.setStates(states);
        }

        return tabAccessFeedbackMapper.getTabAccessFeedbackList(tabAccessFeedback);
    }

    @Override
    public AjaxResult getTabAccessFeedbackById(Long id) {
        TabAccessFeedback byId = this.getById(id);

        if (byId == null) {
            return AjaxResult.success("查询失败");
        }

        return AjaxResult.success(byId);
    }


    @Override
    @Transactional
    public AjaxResult insertTabAccessFeedback(List<TabAccessFeedback> tabAccessFeedbacks) {
        tabAccessFeedbacks.forEach(tabAccessFeedback -> {
            Snowflake snowflake = IdUtil.getSnowflake(1, 1);
            long id = snowflake.nextId();
            tabAccessFeedback.setId(id);
            tabAccessFeedback.setCreateBy(getUsername());
            tabAccessFeedback.setCreateTime(new Date());
            tabAccessFeedback.setUserId(getUserId());
            tabAccessFeedback.setDeptId(getDeptId());
        });

        boolean b = this.saveBatch(tabAccessFeedbacks);
        if (b) {
            return AjaxResult.success("添加成功");
        } else {
            return AjaxResult.success("添加失败");
        }
    }

    @Override
    @Transactional
    public AjaxResult updateTabAccessFeedbackById(List<TabAccessFeedback> tabAccessFeedbacks) {

        tabAccessFeedbacks.forEach(tabAccessFeedback -> {
            tabAccessFeedback.setUpdateBy(getUsername());
            tabAccessFeedback.setUpdateTime(new Date());
        });
        boolean b = this.saveOrUpdateBatch(tabAccessFeedbacks);
        if (b) {
            return AjaxResult.success("修改成功");
        } else {
            return AjaxResult.success("修改失败");
        }
    }

    @Override
    @Transactional
    public AjaxResult falseDeleteTabAccessFeedbackById(Long id) {
        TabAccessFeedback tabAccessFeedback = new TabAccessFeedback();
        tabAccessFeedback.setId(id);
        tabAccessFeedback.setIsDel(1);
        boolean b = this.updateById(tabAccessFeedback);
        if (b) {
            return AjaxResult.success("删除成功");
        } else {
            return AjaxResult.success("删除失败");
        }
    }

    @Override
    public AjaxResult getTabAccessFeedbackListByAccessOnlyId(Long accessOnlyId) {
        LambdaQueryWrapper<TabAccessFeedback> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TabAccessFeedback::getAccessOnlyId, accessOnlyId);
        return AjaxResult.success(this.list(queryWrapper));
    }
}