package com.citygis.web.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.citygis.common.annotation.DataSource;
import com.citygis.common.core.domain.AjaxResult;
import com.citygis.common.enums.DataSourceType;
import com.citygis.framework.datasource.DynamicDataSourceContextHolder;
import com.citygis.web.domain.*;
import com.citygis.web.mapper.TabDatasourceMsgMapper;
import com.citygis.web.service.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 接入数据资源详情 服务实现类
 * </p>
 *
 * @author citygis
 * @since 2025-01-08
 */
@Service
public class TabDatasourceMsgServiceImpl extends ServiceImpl<TabDatasourceMsgMapper, TabDatasourceMsg> implements ITabDatasourceMsgService {


    @Resource
    ITabEtlService tabEtlService;

    @Resource
    ITabKafkaService tabKafkaService;

    @Resource
    ITabKbService tabKbService;

    @Resource
    ITabSftpService tabSftpService;

    @Resource
    ITabUrlService tabUrlService;

    @Resource
    ITabSystemMsgService tabSystemMsgService;

    @Override
    @Transactional
    public AjaxResult insertDataSun(DataSun dataSun) {
        Snowflake snowflake = IdUtil.getSnowflake(1, 1);

        // 统一处理各类数据
        saveAndCreateTabSystemMsg(dataSun.getEtl(), "ETL", dataSun.getAccessFeedbackId(), snowflake);
        saveAndCreateTabSystemMsg(dataSun.getKafka(), "KAFKA", dataSun.getAccessFeedbackId(), snowflake);
        saveAndCreateTabSystemMsg(dataSun.getKb(), "KB", dataSun.getAccessFeedbackId(), snowflake);
        saveAndCreateTabSystemMsg(dataSun.getSftp(), "SFTP", dataSun.getAccessFeedbackId(), snowflake);
        saveAndCreateTabSystemMsg(dataSun.getUrl(), "URL", dataSun.getAccessFeedbackId(), snowflake);

        return AjaxResult.success("添加成功");
    }

    // 通用方法：处理保存和 TabSystemMsg 创建
    private <T> void saveAndCreateTabSystemMsg(List<T> dataList, String accessManner, Long accessFeedbackId, Snowflake snowflake) {
        if (dataList != null) {
            dataList.forEach(data -> {
                // 设置反馈ID
                setAccessFeedbackId(data, accessFeedbackId);

                // 根据不同类型的实体进行保存
                if (data instanceof TabEtl) {
                    ((TabEtl) data).setId(snowflake.nextId());
                    tabEtlService.save((TabEtl) data);
                } else if (data instanceof TabKafka) {
                    ((TabKafka) data).setId(snowflake.nextId());
                    tabKafkaService.save((TabKafka) data);
                } else if (data instanceof TabKb) {
                    ((TabKb) data).setId(snowflake.nextId());
                    tabKbService.save((TabKb) data);
                } else if (data instanceof TabSftp) {
                    ((TabSftp) data).setId(snowflake.nextId());
                    tabSftpService.save((TabSftp) data);
                } else if (data instanceof TabUrl) {
                    ((TabUrl) data).setId(snowflake.nextId());
                    tabUrlService.save((TabUrl) data);
                }

                // 创建并保存 TabSystemMsg
//                 createTabSystemMsg(data, accessManner, snowflake);


            });
        }
    }



    // 创建 TabSystemMsg
    @DataSource(DataSourceType.SLAVE)
    private <T> void createTabSystemMsg(T data, String accessManner, Snowflake snowflake) {
        TabSystemMsg tsm = new TabSystemMsg();
        tsm.setId((int) snowflake.nextId());

        if (data instanceof TabEtl) {
            tsm.setTableEnglishName(((TabEtl) data).getTableEnglishName());
            tsm.setTableName(((TabEtl) data).getTableName());
            tsm.setSystemName(((TabEtl) data).getScenario());
            tsm.setDataEssentialId(((TabEtl) data).getId());
        } else if (data instanceof TabKafka) {
            tsm.setTableEnglishName(((TabKafka) data).getTopicEnglishName());
            tsm.setTableName(((TabKafka) data).getTopicName());
            tsm.setSystemName(((TabKafka) data).getScenario());
            tsm.setDataEssentialId(((TabKafka) data).getId());
        } else if (data instanceof TabKb) {
            tsm.setTableEnglishName(((TabKb) data).getTableEnglishName());
            tsm.setTableName(((TabKb) data).getTableName());
            tsm.setSystemName(((TabKb) data).getScenario());
            tsm.setDataEssentialId(((TabKb) data).getId());
        } else if (data instanceof TabSftp) {
            tsm.setTableEnglishName(((TabSftp) data).getFileEnglishName());
            tsm.setTableName(((TabSftp) data).getFileName());
            tsm.setSystemName(((TabSftp) data).getScenario());
            tsm.setDataEssentialId(((TabSftp) data).getId());
        } else if (data instanceof TabUrl) {
            tsm.setTableEnglishName(((TabUrl) data).getOfferUrl());
            tsm.setTableName(((TabUrl) data).getUrlDetail());
            tsm.setSystemName(((TabUrl) data).getScenario());
            tsm.setDataEssentialId(((TabUrl) data).getId());
        }

        tsm.setAccessManner(accessManner);
        DynamicDataSourceContextHolder.setDataSourceType(DataSourceType.SLAVE.name());
        boolean b = tabSystemMsgService.save(tsm);
        DynamicDataSourceContextHolder.clearDataSourceType();;
    }


    @Override
    @Transactional
    public AjaxResult updateDataSun(DataSun dataSun) {
        updateTabSystemMsg(dataSun.getEtl(), "ETL", dataSun.getAccessFeedbackId());
        updateTabSystemMsg(dataSun.getKafka(), "KAFKA", dataSun.getAccessFeedbackId());
        updateTabSystemMsg(dataSun.getKb(), "KB", dataSun.getAccessFeedbackId());
        updateTabSystemMsg(dataSun.getSftp(), "SFTP", dataSun.getAccessFeedbackId());
        updateTabSystemMsg(dataSun.getUrl(), "URL", dataSun.getAccessFeedbackId());

        return AjaxResult.success("修改成功");
    }

    private <T> void updateTabSystemMsg(List<T> dataList, String accessManner, Long accessFeedbackId) {
        if (dataList != null) {
            dataList.forEach(data -> {
                // 设置反馈ID
                setAccessFeedbackId(data, accessFeedbackId);

                // 根据数据类型保存或更新
                if (data instanceof TabEtl) {
                    tabEtlService.saveOrUpdate((TabEtl) data);
                } else if (data instanceof TabKafka) {
                    tabKafkaService.saveOrUpdate((TabKafka) data);
                } else if (data instanceof TabKb) {
                    tabKbService.saveOrUpdate((TabKb) data);
                } else if (data instanceof TabSftp) {
                    tabSftpService.saveOrUpdate((TabSftp) data);
                } else if (data instanceof TabUrl) {
                    tabUrlService.saveOrUpdate((TabUrl) data);
                }

//                // 更新或保存 TabSystemMsg
//                LambdaQueryWrapper<TabSystemMsg> updateWrapper = new LambdaQueryWrapper<>();
//                if (data instanceof TabEtl) {
//                    updateWrapper.eq(TabSystemMsg::getDataEssentialId, ((TabEtl) data).getId());
//                } else if (data instanceof TabKafka) {
//                    updateWrapper.eq(TabSystemMsg::getDataEssentialId, ((TabKafka) data).getId());
//                } else if (data instanceof TabKb) {
//                    updateWrapper.eq(TabSystemMsg::getDataEssentialId, ((TabKb) data).getId());
//                } else if (data instanceof TabSftp) {
//                    updateWrapper.eq(TabSystemMsg::getDataEssentialId, ((TabSftp) data).getId());
//                } else if (data instanceof TabUrl) {
//                    updateWrapper.eq(TabSystemMsg::getDataEssentialId, ((TabUrl) data).getId());
//                }
//
//                // 检查是否已经存在
//                TabSystemMsg one = tabSystemMsgService.getOne(updateWrapper);
//
//                // 创建 TabSystemMsg 对象
//                TabSystemMsg tsm = createTabSystemMsg(data, accessManner);
//
//                // 根据是否存在记录决定是插入还是更新
//                if (one != null) {
//                    tabSystemMsgService.update(tsm, updateWrapper);
//                } else {
//                    tabSystemMsgService.save(tsm);
//                }
            });
        }
    }

    private <T> void setAccessFeedbackId(T data, Long accessFeedbackId) {
        if (data instanceof TabEtl) {
            ((TabEtl) data).setAccessFeedbackId(accessFeedbackId);
        } else if (data instanceof TabKafka) {
            ((TabKafka) data).setAccessFeedbackId(accessFeedbackId);
        } else if (data instanceof TabKb) {
            ((TabKb) data).setAccessFeedbackId(accessFeedbackId);
        } else if (data instanceof TabSftp) {
            ((TabSftp) data).setAccessFeedbackId(accessFeedbackId);
        } else if (data instanceof TabUrl) {
            ((TabUrl) data).setAccessFeedbackId(accessFeedbackId);
        }
    }

    private <T> TabSystemMsg createTabSystemMsg(T data, String accessManner) {
        TabSystemMsg tsm = new TabSystemMsg();

        if (data instanceof TabEtl) {
            tsm.setTableEnglishName(((TabEtl) data).getTableEnglishName());
            tsm.setTableName(((TabEtl) data).getTableName());
            tsm.setSystemName(((TabEtl) data).getScenario());
        } else if (data instanceof TabKafka) {
            tsm.setTableEnglishName(((TabKafka) data).getTopicEnglishName());
            tsm.setTableName(((TabKafka) data).getTopicName());
            tsm.setSystemName(((TabKafka) data).getScenario());
        } else if (data instanceof TabKb) {
            tsm.setTableEnglishName(((TabKb) data).getTableEnglishName());
            tsm.setTableName(((TabKb) data).getTableName());
            tsm.setSystemName(((TabKb) data).getScenario());
        } else if (data instanceof TabSftp) {
            tsm.setTableEnglishName(((TabSftp) data).getFileEnglishName());
            tsm.setTableName(((TabSftp) data).getFileName());
            tsm.setSystemName(((TabSftp) data).getScenario());
        } else if (data instanceof TabUrl) {
            tsm.setTableEnglishName(((TabUrl) data).getOfferUrl());
            tsm.setTableName(((TabUrl) data).getUrlDetail());
            tsm.setSystemName(((TabUrl) data).getScenario());
        }

        tsm.setAccessManner(accessManner);
        return tsm;
    }


    @Override
    @Transactional
    public AjaxResult deleteDataSun(Long id) {
        // 更新 TabEtl 的删除状态
        updateTabEntityStatus(TabEtl.class, id);

        // 更新 TabKafka 的删除状态
        updateTabEntityStatus(TabKafka.class, id);

        // 更新 TabKb 的删除状态
        updateTabEntityStatus(TabKb.class, id);

        // 更新 TabSftp 的删除状态
        updateTabEntityStatus(TabSftp.class, id);

        // 更新 TabUrl 的删除状态
        updateTabEntityStatus(TabUrl.class, id);

        return AjaxResult.success("删除成功");
    }

    // 通用方法：更新 TabEntity 的删除状态，并更新相关的 TabSystemMsg
    private <T> void updateTabEntityStatus(Class<T> entityClass, Long id) {
        // 创建对应的实体对象并更新删除状态
        try {
            T entity = entityClass.getDeclaredConstructor().newInstance();
            entityClass.getMethod("setAccessFeedbackId", Long.class).invoke(entity, id);
            entityClass.getMethod("setIsDel", Integer.class).invoke(entity, 1);
            Object service = getServiceByEntityClass(entityClass);
            service.getClass().getMethod("updateById", entityClass).invoke(service, entity);

//            // 查询 TabSystemMsg 相关记录并更新删除状态
//            LambdaQueryWrapper<TabSystemMsg> queryWrapper = new LambdaQueryWrapper<>();
//            queryWrapper.eq(TabSystemMsg::getDataEssentialId, id);
//            TabSystemMsg systemMsg = tabSystemMsgService.getOne(queryWrapper);
//            if (systemMsg != null) {
//                systemMsg.setIsDel(1);
//                tabSystemMsgService.update(systemMsg, queryWrapper);
//            }
        } catch (Exception e) {
            e.printStackTrace(); // 你可以根据需要处理异常
        }
    }

    // 根据实体类类型获取对应的服务
    private Object getServiceByEntityClass(Class<?> entityClass) {
        if (entityClass.equals(TabEtl.class)) {
            return tabEtlService;
        } else if (entityClass.equals(TabKafka.class)) {
            return tabKafkaService;
        } else if (entityClass.equals(TabKb.class)) {
            return tabKbService;
        } else if (entityClass.equals(TabSftp.class)) {
            return tabSftpService;
        } else if (entityClass.equals(TabUrl.class)) {
            return tabUrlService;
        }
        return null;
    }


    @Override
    public AjaxResult getDataSun(Long id) {
        LambdaQueryWrapper<TabEtl> ql = new LambdaQueryWrapper<>();
        ql.eq(TabEtl::getAccessFeedbackId, id);
        List<TabEtl> tabEtls = tabEtlService.list(ql);

        LambdaQueryWrapper<TabKafka> kql = new LambdaQueryWrapper<>();
        kql.eq(TabKafka::getAccessFeedbackId, id);
        List<TabKafka> tabKafkas = tabKafkaService.list(kql);

        LambdaQueryWrapper<TabKb> kbql = new LambdaQueryWrapper<>();
        kbql.eq(TabKb::getAccessFeedbackId, id);
        List<TabKb> tabKbs = tabKbService.list(kbql);

        LambdaQueryWrapper<TabSftp> sftpql = new LambdaQueryWrapper<>();
        sftpql.eq(TabSftp::getAccessFeedbackId, id);
        List<TabSftp> tabSftps = tabSftpService.list(sftpql);

        LambdaQueryWrapper<TabUrl> urlql = new LambdaQueryWrapper<>();
        urlql.eq(TabUrl::getAccessFeedbackId, id);
        List<TabUrl> tabUrls = tabUrlService.list(urlql);

        DataSun dataSun = new DataSun();
        dataSun.setEtl(tabEtls);
        dataSun.setKafka(tabKafkas);
        dataSun.setKb(tabKbs);
        dataSun.setSftp(tabSftps);
        dataSun.setUrl(tabUrls);

        return AjaxResult.success(dataSun);
    }


}
